"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const marked_ts_1 = require("marked-ts");
const path = require("path");
const readline = require("readline");
const highlight_js_1 = require("highlight.js");
marked_ts_1.Marked.setOptions({ highlight: (code, lang) => lang == null ? code : highlight_js_1.highlight(lang, code).value });
exports.recursiveMd2Html = (dir, ext, callback) => {
    fs.readdir(dir, (err, files) => {
        if (err)
            return callback(err);
        let pending = files.length;
        if (!pending)
            return callback(void 0);
        files.forEach((fname) => {
            fname = path.resolve(dir, fname);
            fs.stat(fname, (error, stats) => {
                if (error != null)
                    return callback(error);
                else if (stats && stats.isDirectory()) {
                    if (['node_modules', '.git',].indexOf(path.basename(fname)) > -1)
                        /* tslint:disable:no-unused-expression */ !--pending && callback(void 0);
                    else
                        exports.recursiveMd2Html(fname, ext, (e) => {
                            if (e != null)
                                return callback(e);
                            if (!--pending)
                                callback(void 0);
                        });
                }
                else if (fname.length >= ext.length && fname.slice(fname.length - ext.length) === ext
                    && ['README.md', 'LICENSE.md', 'CHANGELOG.md'].indexOf(path.basename(fname)) === -1)
                    parseTemplateUrl(fname.split('').reverse().join('')
                        .replace(ext.split('').reverse().join(''), 'st.').split('').reverse().join(''), (er, templateUrl) => {
                        if (er != null)
                            return callback(er.errno === -2 ? void 0 : er);
                        else if (templateUrl == null || !templateUrl.length || !templateUrl.endsWith(ext))
                            return callback();
                        handleMarkdown(fname, path.join(path.dirname(fname), templateUrl), (e_r, to_fname) => {
                            if (e_r != null)
                                return callback(e_r);
                            escapeBrace(to_fname, ext, erro => {
                                if (erro != null)
                                    return callback(erro);
                                console.info(`Generated:\t ${to_fname}`);
                                if (!--pending)
                                    callback(void 0);
                            });
                        });
                    });
            });
        });
    });
};
const neg1_to_max = (n) => n === -1 ? Number.MAX_SAFE_INTEGER : n;
// *VERY* basic parser. Doesn't handle comments. In fact, you are recommended to set `templateUrl` in a comment!
const parseTemplateUrl = (fname, callback) => {
    fs.readFile(fname, 'utf8', (err, data) => {
        if (err != null)
            return callback(err);
        if (data.indexOf('@Component') === -1)
            return callback(void 0); // We're not in a component
        const fst = data.indexOf('templateUrl');
        if (fst === -1)
            return callback(void 0);
        const j = data.slice(fst + 'templateUrl'.length + 3);
        return callback(void 0, j.slice(0, ['\'', '"', '`']
            .map(c => neg1_to_max(j.indexOf(c)))
            .reduce((a, b) => a > b ? b : a, Number.MAX_SAFE_INTEGER)));
    });
};
const handleMarkdown = (fname, templateUrl, callback) => {
    fs.readFile(templateUrl, 'utf8', (err, data) => {
        if (err != null)
            callback(err);
        const to_fname = `${templateUrl.substr(0, templateUrl.length - 3)}.html`;
        fs.writeFile(to_fname, marked_ts_1.Marked.parse(data), 'utf8', e => callback(e, to_fname));
    });
};
const escapeBrace = (fname, ext, callback) => {
    // TODO: Character by character parser, to support one line blocks
    const lineReader = readline.createInterface({
        input: fs.createReadStream(fname)
    });
    const lines = [
        `<!-- GENERATED FROM './${path.basename(fname).slice(0, -5)}${ext}'. ` +
            'EDIT THAT, XOR DELETE AND EDIT THIS! -->\n'
    ];
    let code_block = 0;
    lineReader.on('line', (line) => {
        /* tslint:disable:no-bitwise */
        /*if ((code_block & 1) !== 0)
            line = line.replace(/{/g, `{{'{'}}`);*/
        code_block += ['<code', '</code'].reduce((a, b) => a + line.indexOf(b) > -1, 0);
        lines.push(line);
    });
    lineReader.on('close', () => fs.writeFile(fname, lines.join('\n').replace(/{/g, '&#0123;'), callback));
};
